# -*- coding: utf-8 -*-
################################################################################
#
#  pyCatapult -- Python Interface to ECRS Catapult
#  Copyright © 2016 Lance Edgar
#
#  This file is part of pyCatapult.
#
#  pyCatapult is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  pyCatapult is distributed in the hope that it will be useful, but WITHOUT
#  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#  more details.
#
#  You should have received a copy of the GNU General Public License along with
#  pyCatapult.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Tweaks for Sybase dialect used with SQLAlchemy

As of SQLAlchemy 1.1 their default Sybase dialect raises an exception whenever
an attempt is made to fetch an 'offset' query (used in pagination scenarios).
This dialect effectively forks their default slightly, to restore the previous
'offset' behavior.
"""

from __future__ import unicode_literals, absolute_import

from sqlalchemy.dialects import sybase


class CatapultSQLCompiler(sybase.dialect.statement_compiler):

    def get_select_precolumns(self, select, **kw):
        """
        .. note::
           The code for this method was copied directly from that found in
           SQLAlchemy 1.0.15
        """
        s = select._distinct and "DISTINCT " or ""
        # TODO: don't think Sybase supports
        # bind params for FIRST / TOP
        limit = select._limit
        if limit:
            # if select._limit == 1:
                # s += "FIRST "
            # else:
                # s += "TOP %s " % (select._limit,)
            s += "TOP %s " % (limit,)
        offset = select._offset
        if offset:
            if not limit:
                # FIXME: sybase doesn't allow an offset without a limit
                # so use a huge value for TOP here
                s += "TOP 1000000 "
            s += "START AT %s " % (offset + 1,)
        return s


class CatapultDialect(sybase.dialect):
    """
    Overrides default 'sybase' dialect behavior slightly.
    """
    name = 'catapult'
    statement_compiler = CatapultSQLCompiler
